@@ -24,9 +24,7 @@ import daemon
 import psutil
 import sqlalchemy.exc
 from celery import maybe_patch_concurrency
-from celery.bin import worker as worker_bin
 from daemon.pidfile import TimeoutPIDLockFile
-from flower.command import FlowerCommand
 from lockfile.pidlockfile import read_pid_from_pidfile, remove_existing_pidfile
 
 from airflow import settings
@@ -43,6 +41,7 @@ WORKER_PROCESS_NAME = "worker"
 def flower(args):
     """Starts Flower, Celery monitoring tool"""
     options = [
+        "flower",
         conf.get('celery', 'BROKER_URL'),
         f"--address={args.hostname}",
         f"--port={args.port}",
@@ -60,8 +59,6 @@ def flower(args):
     if args.flower_conf:
         options.append(f"--conf={args.flower_conf}")
 
-    flower_cmd = FlowerCommand()
-
     if args.daemon:
         pidfile, stdout, stderr, _ = setup_locations(
             process="flower",
@@ -77,9 +74,9 @@ def flower(args):
                 stderr=stderr,
             )
             with ctx:
-                flower_cmd.execute_from_commandline(argv=options)
+                celery_app.start(options)
     else:
-        flower_cmd.execute_from_commandline(argv=options)
+        celery_app.start(options)
 
 
 def _serve_logs(skip_serve_logs: bool = False) -> Optional[Process]:
@@ -91,6 +88,15 @@ def _serve_logs(skip_serve_logs: bool = False) -> Optional[Process]:
     return None
 
 
+def _run_worker(options, skip_serve_logs):
+    sub_proc = _serve_logs(skip_serve_logs)
+    try:
+        celery_app.worker_main(options)
+    finally:
+        if sub_proc:
+            sub_proc.terminate()
+
+
 @cli_utils.action_logging
 def worker(args):
     """Starts Airflow Celery worker"""
@@ -129,23 +135,31 @@ def worker(args):
             pass
 
     # Setup Celery worker
-    worker_instance = worker_bin.worker(app=celery_app)
-    options = {
-        'optimization': 'fair',
-        'O': 'fair',
-        'queues': args.queues,
-        'concurrency': args.concurrency,
-        'autoscale': autoscale,
-        'hostname': args.celery_hostname,
-        'loglevel': conf.get('logging', 'LOGGING_LEVEL'),
-        'pidfile': pid_file_path,
-        'without_mingle': args.without_mingle,
-        'without_gossip': args.without_gossip,
-    }
+    options = [
+        'worker',
+        '-O',
+        'fair',
+        '--queues',
+        args.queues,
+        '--concurrency',
+        args.concurrency,
+        '--hostname',
+        args.celery_hostname,
+        '--loglevel',
+        conf.get('logging', 'LOGGING_LEVEL'),
+        '--pidfile',
+        pid_file_path,
+    ]
+    if autoscale:
+        options.extend(['--autoscale', autoscale])
+    if args.without_mingle:
+        options.append('--without-mingle')
+    if args.without_gossip:
+        options.append('--without-gossip')
 
     if conf.has_option("celery", "pool"):
         pool = conf.get("celery", "pool")
-        options["pool"] = pool
+        options.extend(["--pool", pool])
         # Celery pools of type eventlet and gevent use greenlets, which
         # requires monkey patching the app:
         # https://eventlet.net/doc/patching.html#monkey-patch
@@ -168,22 +182,20 @@ def worker(args):
                 stderr=stderr_handle,
             )
             with ctx:
-                sub_proc = _serve_logs(skip_serve_logs)
-                worker_instance.run(**options)
+                _run_worker(options=options, skip_serve_logs=skip_serve_logs)
     else:
         # Run Celery worker in the same process
-        sub_proc = _serve_logs(skip_serve_logs)
-        worker_instance.run(**options)
-
-    if sub_proc:
-        sub_proc.terminate()
+        _run_worker(options=options, skip_serve_logs=skip_serve_logs)
 
 
 @cli_utils.action_logging
-def stop_worker(args):  # pylint: disable=unused-argument
+def stop_worker(args):
     """Sends SIGTERM to Celery worker"""
     # Read PID from file
-    pid_file_path, _, _, _ = setup_locations(process=WORKER_PROCESS_NAME)
+    if args.pid:
+        pid_file_path = args.pid
+    else:
+        pid_file_path, _, _, _ = setup_locations(process=WORKER_PROCESS_NAME)
     pid = read_pid_from_pidfile(pid_file_path)
 
     # Send SIGTERM
