@@ -21,17 +21,21 @@ import json
 import logging
 import os
 import textwrap
-from contextlib import contextmanager, redirect_stderr, redirect_stdout
-from typing import List
+from contextlib import contextmanager, redirect_stderr, redirect_stdout, suppress
+from typing import List, Optional
+
+from pendulum.parsing.exceptions import ParserError
+from sqlalchemy.orm.exc import NoResultFound
 
 from airflow import settings
 from airflow.cli.simple_table import AirflowConsole
 from airflow.configuration import conf
-from airflow.exceptions import AirflowException
+from airflow.exceptions import AirflowException, DagRunNotFound
 from airflow.executors.executor_loader import ExecutorLoader
 from airflow.jobs.local_task_job import LocalTaskJob
 from airflow.models import DagPickle, TaskInstance
 from airflow.models.dag import DAG
+from airflow.models.dagrun import DagRun
 from airflow.ti_deps.dep_context import DepContext
 from airflow.ti_deps.dependencies_deps import SCHEDULER_QUEUED_DEPS
 from airflow.utils import cli as cli_utils
@@ -42,9 +46,51 @@ from airflow.utils.cli import (
     get_dags,
     suppress_logs_and_warning,
 )
+from airflow.utils.dates import timezone
 from airflow.utils.log.logging_mixin import StreamLogWriter
 from airflow.utils.net import get_hostname
-from airflow.utils.session import create_session
+from airflow.utils.session import create_session, provide_session
+
+
+def _get_dag_run(dag, exec_date_or_run_id, create_if_necssary, session):
+    dag_run = dag.get_dagrun(run_id=exec_date_or_run_id, session=session)
+    if dag_run:
+        return dag_run
+
+    execution_date = None
+    with suppress(ParserError, TypeError):
+        execution_date = timezone.parse(exec_date_or_run_id)
+
+    if create_if_necssary and not execution_date:
+        return DagRun(dag_id=dag.dag_id, run_id=exec_date_or_run_id)
+    try:
+        return (
+            session.query(DagRun)
+            .filter(
+                DagRun.dag_id == dag.dag_id,
+                DagRun.execution_date == execution_date,
+            )
+            .one()
+        )
+    except NoResultFound:
+        if create_if_necssary:
+            return DagRun(dag.dag_id, execution_date=execution_date)
+        raise DagRunNotFound(
+            f"DagRun for {dag.dag_id} with run_id or execution_date of {exec_date_or_run_id!r} not found"
+        ) from None
+
+
+@provide_session
+def _get_ti(task, exec_date_or_run_id, create_if_necssary=False, session=None):
+    """Get the task instance through DagRun.run_id, if that fails, get the TI the old way"""
+    dag_run = _get_dag_run(task.dag, exec_date_or_run_id, create_if_necssary, session)
+
+    ti = dag_run.get_task_instance(task.task_id)
+    if not ti and create_if_necssary:
+        ti = TaskInstance(task, run_id=None)
+        ti.dag_run = dag_run
+    ti.refresh_from_task(task)
+    return ti
 
 
 def _run_task_by_selected_method(args, dag: DAG, ti: TaskInstance) -> None:
@@ -55,11 +101,6 @@ def _run_task_by_selected_method(args, dag: DAG, ti: TaskInstance) -> None:
     - as raw task
     - by executor
     """
-    if args.local and args.raw:
-        raise AirflowException(
-            "Option --raw and --local are mutually exclusive. "
-            "Please remove one option to execute the command."
-        )
     if args.local:
         _run_task_by_local_task_job(args, ti)
     elif args.raw:
@@ -80,14 +121,15 @@ def _run_task_by_executor(args, dag, ti):
             with create_session() as session:
                 pickle = DagPickle(dag)
                 session.add(pickle)
-                pickle_id = pickle.id
-                # TODO: This should be written to a log
-                print(f'Pickled dag {dag} as pickle_id: {pickle_id}')
+            pickle_id = pickle.id
+            # TODO: This should be written to a log
+            print(f'Pickled dag {dag} as pickle_id: {pickle_id}')
         except Exception as e:
             print('Could not pickle the DAG')
             print(e)
             raise e
     executor = ExecutorLoader.get_default_executor()
+    executor.job_id = "manual"
     executor.start()
     print("Sending to executor.")
     executor.queue_task_instance(
@@ -115,6 +157,7 @@ def _run_task_by_local_task_job(args, ti):
         ignore_task_deps=args.ignore_dependencies,
         ignore_ti_state=args.force,
         pool=args.pool,
+        external_executor_id=_extract_external_executor_id(args),
     )
     try:
         run_job.run()
@@ -134,18 +177,7 @@ RAW_TASK_UNSUPPORTED_OPTION = [
 
 def _run_raw_task(args, ti: TaskInstance) -> None:
     """Runs the main task handling code"""
-    unsupported_options = [o for o in RAW_TASK_UNSUPPORTED_OPTION if getattr(args, o)]
-
-    if unsupported_options:
-        raise AirflowException(
-            "Option --raw does not work with some of the other options on this command. You "
-            "can't use --raw option and the following options: {}. You provided the option {}. "
-            "Delete it to execute the command".format(
-                ", ".join(f"--{o}" for o in RAW_TASK_UNSUPPORTED_OPTION),
-                ", ".join(f"--{o}" for o in unsupported_options),
-            )
-        )
-    ti._run_raw_task(  # pylint: disable=protected-access
+    ti._run_raw_task(
         mark_success=args.mark_success,
         job_id=args.job_id,
         pool=args.pool,
@@ -153,6 +185,12 @@ def _run_raw_task(args, ti: TaskInstance) -> None:
     )
 
 
+def _extract_external_executor_id(args) -> Optional[str]:
+    if hasattr(args, "external_executor_id"):
+        return getattr(args, "external_executor_id")
+    return os.environ.get("external_executor_id", None)
+
+
 @contextmanager
 def _capture_task_logs(ti):
     """Manage logging context for a task run
@@ -192,6 +230,27 @@ def _capture_task_logs(ti):
 def task_run(args, dag=None):
     """Runs a single task instance"""
     # Load custom airflow config
+
+    if args.local and args.raw:
+        raise AirflowException(
+            "Option --raw and --local are mutually exclusive. "
+            "Please remove one option to execute the command."
+        )
+
+    if args.raw:
+        unsupported_options = [o for o in RAW_TASK_UNSUPPORTED_OPTION if getattr(args, o)]
+
+        if unsupported_options:
+            raise AirflowException(
+                "Option --raw does not work with some of the other options on this command. You "
+                "can't use --raw option and the following options: {}. You provided the option {}. "
+                "Delete it to execute the command".format(
+                    ", ".join(f"--{o}" for o in RAW_TASK_UNSUPPORTED_OPTION),
+                    ", ".join(f"--{o}" for o in unsupported_options),
+                )
+            )
+    if dag and args.pickle:
+        raise AirflowException("You cannot use the --pickle option when using DAG.cli() method.")
     if args.cfg_path:
         with open(args.cfg_path) as conf_file:
             conf_dict = json.load(conf_file)
@@ -210,9 +269,7 @@ def task_run(args, dag=None):
     # processing hundreds of simultaneous tasks.
     settings.configure_orm(disable_connection_pool=True)
 
-    if dag and args.pickle:
-        raise AirflowException("You cannot use the --pickle option when using DAG.cli() method.")
-    elif args.pickle:
+    if args.pickle:
         print(f'Loading pickle id: {args.pickle}')
         dag = get_dag_by_pickle(args.pickle)
     elif not dag:
@@ -220,10 +277,8 @@ def task_run(args, dag=None):
     else:
         # Use DAG from parameter
         pass
-
     task = dag.get_task(task_id=args.task_id)
-    ti = TaskInstance(task, args.execution_date)
-    ti.refresh_from_db()
+    ti = _get_ti(task, args.execution_date_or_run_id)
     ti.init_run_context(raw=args.raw)
 
     hostname = get_hostname()
@@ -251,7 +306,7 @@ def task_failed_deps(args):
     """
     dag = get_dag(args.subdir, args.dag_id)
     task = dag.get_task(task_id=args.task_id)
-    ti = TaskInstance(task, args.execution_date)
+    ti = _get_ti(task, args.execution_date_or_run_id)
 
     dep_context = DepContext(deps=SCHEDULER_QUEUED_DEPS)
     failed_deps = list(ti.get_failed_dep_statuses(dep_context=dep_context))
@@ -274,7 +329,7 @@ def task_state(args):
     """
     dag = get_dag(args.subdir, args.dag_id)
     task = dag.get_task(task_id=args.task_id)
-    ti = TaskInstance(task, args.execution_date)
+    ti = _get_ti(task, args.execution_date_or_run_id)
     print(ti.current_state())
 
 
@@ -286,7 +341,7 @@ def task_list(args, dag=None):
     if args.tree:
         dag.tree_view()
     else:
-        tasks = sorted([t.task_id for t in dag.tasks])
+        tasks = sorted(t.task_id for t in dag.tasks)
         print("\n".join(tasks))
 
 
@@ -320,37 +375,43 @@ def _guess_debugger():
 
 @cli_utils.action_logging
 @suppress_logs_and_warning
-def task_states_for_dag_run(args):
+@provide_session
+def task_states_for_dag_run(args, session=None):
     """Get the status of all task instances in a DagRun"""
-    with create_session() as session:
-        tis = (
-            session.query(
-                TaskInstance.dag_id,
-                TaskInstance.execution_date,
-                TaskInstance.task_id,
-                TaskInstance.state,
-                TaskInstance.start_date,
-                TaskInstance.end_date,
+    dag_run = (
+        session.query(DagRun)
+        .filter(DagRun.run_id == args.execution_date_or_run_id, DagRun.dag_id == args.dag_id)
+        .one_or_none()
+    )
+    if not dag_run:
+        try:
+            execution_date = timezone.parse(args.execution_date_or_run_id)
+            dag_run = (
+                session.query(DagRun)
+                .filter(DagRun.execution_date == execution_date, DagRun.dag_id == args.dag_id)
+                .one_or_none()
             )
-            .filter(TaskInstance.dag_id == args.dag_id, TaskInstance.execution_date == args.execution_date)
-            .all()
+        except (ParserError, TypeError) as err:
+            raise AirflowException(f"Error parsing the supplied execution_date. Error: {str(err)}")
+
+    if dag_run is None:
+        raise DagRunNotFound(
+            f"DagRun for {args.dag_id} with run_id or execution_date of {args.execution_date_or_run_id!r} "
+            "not found"
         )
 
-        if len(tis) == 0:
-            raise AirflowException("DagRun does not exist.")
-
-        AirflowConsole().print_as(
-            data=tis,
-            output=args.output,
-            mapper=lambda ti: {
-                "dag_id": ti.dag_id,
-                "execution_date": ti.execution_date.isoformat(),
-                "task_id": ti.task_id,
-                "state": ti.state,
-                "start_date": ti.start_date.isoformat() if ti.start_date else "",
-                "end_date": ti.end_date.isoformat() if ti.end_date else "",
-            },
-        )
+    AirflowConsole().print_as(
+        data=dag_run.task_instances,
+        output=args.output,
+        mapper=lambda ti: {
+            "dag_id": ti.dag_id,
+            "execution_date": dag_run.execution_date.isoformat(),
+            "task_id": ti.task_id,
+            "state": ti.state,
+            "start_date": ti.start_date.isoformat() if ti.start_date else "",
+            "end_date": ti.end_date.isoformat() if ti.end_date else "",
+        },
+    )
 
 
 @cli_utils.action_logging
@@ -383,14 +444,18 @@ def task_test(args, dag=None):
     if args.task_params:
         passed_in_params = json.loads(args.task_params)
         task.params.update(passed_in_params)
-    ti = TaskInstance(task, args.execution_date)
+
+    if task.params:
+        task.params.validate()
+
+    ti = _get_ti(task, args.execution_date_or_run_id, create_if_necssary=True)
 
     try:
         if args.dry_run:
             ti.dry_run()
         else:
             ti.run(ignore_task_deps=True, ignore_ti_state=True, test_mode=True)
-    except Exception:  # pylint: disable=broad-except
+    except Exception:
         if args.post_mortem:
             debugger = _guess_debugger()
             debugger.post_mortem()
@@ -409,7 +474,7 @@ def task_render(args):
     """Renders and displays templated fields for a given task"""
     dag = get_dag(args.subdir, args.dag_id)
     task = dag.get_task(task_id=args.task_id)
-    ti = TaskInstance(task, args.execution_date)
+    ti = _get_ti(task, args.execution_date_or_run_id, create_if_necssary=True)
     ti.render_templates()
     for attr in task.__class__.template_fields:
         print(
@@ -429,7 +494,7 @@ def task_clear(args):
     logging.basicConfig(level=settings.LOGGING_LEVEL, format=settings.SIMPLE_LOG_FORMAT)
 
     if args.dag_id and not args.subdir and not args.dag_regex and not args.task_regex:
-        dags = get_dag_by_file_location(args.dag_id)
+        dags = [get_dag_by_file_location(args.dag_id)]
     else:
         # todo clear command only accepts a single dag_id. no reason for get_dags with 's' except regex?
         dags = get_dags(args.subdir, args.dag_id, use_regex=args.dag_regex)
