@@ -18,27 +18,21 @@
 
 """Example DAG demonstrating the usage of the BashOperator."""
 
-from datetime import timedelta
+from datetime import datetime, timedelta
 
 from airflow import DAG
 from airflow.operators.bash import BashOperator
 from airflow.operators.dummy import DummyOperator
-from airflow.utils.dates import days_ago
-
-args = {
-    'owner': 'airflow',
-}
 
 with DAG(
     dag_id='example_bash_operator',
-    default_args=args,
     schedule_interval='0 0 * * *',
-    start_date=days_ago(2),
+    start_date=datetime(2021, 1, 1),
+    catchup=False,
     dagrun_timeout=timedelta(minutes=60),
     tags=['example', 'example2'],
     params={"example_key": "example_value"},
 ) as dag:
-
     run_this_last = DummyOperator(
         task_id='run_this_last',
     )
