@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
@@ -19,26 +18,28 @@
 """
 Example LatestOnlyOperator and TriggerRule interactions
 """
+
+# [START example]
 import datetime as dt
 
-from airflow.models import DAG
-from airflow.operators.dummy_operator import DummyOperator
-from airflow.operators.latest_only_operator import LatestOnlyOperator
-from airflow.utils.dates import days_ago
+from airflow import DAG
+from airflow.operators.dummy import DummyOperator
+from airflow.operators.latest_only import LatestOnlyOperator
 from airflow.utils.trigger_rule import TriggerRule
 
-dag = DAG(
+with DAG(
     dag_id='latest_only_with_trigger',
     schedule_interval=dt.timedelta(hours=4),
-    start_date=days_ago(2),
-    tags=['example']
-)
+    start_date=dt.datetime(2021, 1, 1),
+    catchup=False,
+    tags=['example3'],
+) as dag:
+    latest_only = LatestOnlyOperator(task_id='latest_only')
+    task1 = DummyOperator(task_id='task1')
+    task2 = DummyOperator(task_id='task2')
+    task3 = DummyOperator(task_id='task3')
+    task4 = DummyOperator(task_id='task4', trigger_rule=TriggerRule.ALL_DONE)
 
-latest_only = LatestOnlyOperator(task_id='latest_only', dag=dag)
-task1 = DummyOperator(task_id='task1', dag=dag)
-task2 = DummyOperator(task_id='task2', dag=dag)
-task3 = DummyOperator(task_id='task3', dag=dag)
-task4 = DummyOperator(task_id='task4', dag=dag, trigger_rule=TriggerRule.ALL_DONE)
-
-latest_only >> task1 >> [task3, task4]
-task2 >> [task3, task4]
+    latest_only >> task1 >> [task3, task4]
+    task2 >> [task3, task4]
+# [END example]
