@@ -34,9 +34,9 @@ branch_labels = None
 depends_on = None
 
 
-def upgrade():  # noqa: D103
+def upgrade():
     op.add_column('task_instance', sa.Column('pool_slots', sa.Integer, default=1))
 
 
-def downgrade():  # noqa: D103
+def downgrade():
     op.drop_column('task_instance', 'pool_slots')
