@@ -19,7 +19,7 @@
 """Add dag_hash Column to serialized_dag table
 
 Revision ID: da3f683c3a5a
-Revises: 8d48763f6d53
+Revises: a66efa278eea
 Create Date: 2020-08-07 20:52:09.178296
 
 """
@@ -38,7 +38,8 @@ def upgrade():
     """Apply Add dag_hash Column to serialized_dag table"""
     op.add_column(
         'serialized_dag',
-        sa.Column('dag_hash', sa.String(32), nullable=False, server_default='Hash not calculated yet'))
+        sa.Column('dag_hash', sa.String(32), nullable=False, server_default='Hash not calculated yet'),
+    )
 
 
 def downgrade():
