@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
@@ -26,6 +25,7 @@ from enum import Enum, unique
 @unique
 class Encoding(str, Enum):
     """Enum of encoding constants."""
+
     TYPE = '__type'
     VAR = '__var'
 
@@ -34,6 +34,7 @@ class Encoding(str, Enum):
 @unique
 class DagAttributeTypes(str, Enum):
     """Enum of supported attribute types of DAG."""
+
     DAG = 'dag'
     OP = 'operator'
     DATETIME = 'datetime'
@@ -43,3 +44,7 @@ class DagAttributeTypes(str, Enum):
     DICT = 'dict'
     SET = 'set'
     TUPLE = 'tuple'
+    POD = 'k8s.V1Pod'
+    TASK_GROUP = 'taskgroup'
+    EDGE_INFO = 'edgeinfo'
+    PARAM = 'param'
