@@ -22,21 +22,17 @@ codebase easier.
 """
 
 try:
-    # Protocol and TypedDict are only added to typing module starting from
+    # Literal, Protocol and TypedDict are only added to typing module starting from
     # python 3.8 we can safely remove this shim import after Airflow drops
     # support for <3.8
-    from typing import (  # type: ignore # noqa # pylint: disable=unused-import
-        Protocol,
-        TypedDict,
-        runtime_checkable,
-    )
+    from typing import Literal, Protocol, TypedDict, runtime_checkable  # type: ignore
 except ImportError:
-    from typing_extensions import Protocol, TypedDict, runtime_checkable  # type: ignore # noqa
+    from typing_extensions import Literal, Protocol, TypedDict, runtime_checkable  # type: ignore # noqa
 
 
 # Before Py 3.7, there is no re.Pattern class
 try:
-    from re import Pattern as RePatternType  # type: ignore # pylint: disable=unused-import
+    from re import Pattern as RePatternType  # type: ignore
 except ImportError:
     import re
 
