@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 #
 # Licensed to the Apache Software Foundation (ASF) under one
 # or more contributor license agreements.  See the NOTICE file
@@ -16,28 +15,17 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-#
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
-import re
+import abc
 import logging
+import re
 import sys
-import warnings
-
-import six
-
-from builtins import object
-from contextlib import contextmanager
-from logging import Handler, StreamHandler
+from logging import Handler, Logger, StreamHandler
 
 # 7-bit C1 ANSI escape sequences
 ANSI_ESCAPE = re.compile(r'\x1B[@-_][0-?]*[ -/]*[@-~]')
 
 
-def remove_escape_codes(text):
+def remove_escape_codes(text: str) -> str:
     """
     Remove ANSI escapes codes from string. It's used to remove
     "colors" from log messages.
@@ -45,34 +33,20 @@ def remove_escape_codes(text):
     return ANSI_ESCAPE.sub("", text)
 
 
-class LoggingMixin(object):
-    """
-    Convenience super-class to have a logger configured with the class name
-    """
+class LoggingMixin:
+    """Convenience super-class to have a logger configured with the class name"""
+
     def __init__(self, context=None):
         self._set_context(context)
 
-    # We want to deprecate the logger property in Airflow 2.0
-    # The log property is the de facto standard in most programming languages
     @property
-    def logger(self):
-        warnings.warn(
-            'Initializing logger for {} using logger(), which will '
-            'be replaced by .log in Airflow 2.0'.format(
-                self.__class__.__module__ + '.' + self.__class__.__name__
-            ),
-            DeprecationWarning
-        )
-        return self.log
-
-    @property
-    def log(self):
+    def log(self) -> Logger:
+        """Returns a logger."""
         try:
-            return self._log
+            # FIXME: LoggingMixin should have a default _log field.
+            return self._log  # type: ignore
         except AttributeError:
-            self._log = logging.root.getChild(
-                self.__class__.__module__ + '.' + self.__class__.__name__
-            )
+            self._log = logging.getLogger(self.__class__.__module__ + '.' + self.__class__.__name__)
             return self._log
 
     def _set_context(self, context):
@@ -80,11 +54,29 @@ class LoggingMixin(object):
             set_context(self.log, context)
 
 
-class StreamLogWriter(object):
-    """
-    Allows to redirect stdout and stderr to logger
-    """
-    encoding = None
+class ExternalLoggingMixin:
+    """Define a log handler based on an external service (e.g. ELK, StackDriver)."""
+
+    @property
+    @abc.abstractmethod
+    def log_name(self) -> str:
+        """Return log name"""
+
+    @abc.abstractmethod
+    def get_external_log_url(self, task_instance, try_number) -> str:
+        """Return the URL for log visualization in the external service."""
+
+    @property
+    @abc.abstractmethod
+    def supports_external_link(self) -> bool:
+        """Return whether handler is able to support external links."""
+
+
+# TODO: Formally inherit from io.IOBase
+class StreamLogWriter:
+    """Allows to redirect stdout and stderr to logger"""
+
+    encoding: None = None
 
     def __init__(self, logger, level):
         """
@@ -93,27 +85,33 @@ class StreamLogWriter(object):
         """
         self.logger = logger
         self.level = level
-        self._buffer = str()
+        self._buffer = ''
+
+    def close(self):
+        """
+        Provide close method, for compatibility with the io.IOBase interface.
+
+        This is a no-op method.
+        """
 
     @property
     def closed(self):
         """
-        Returns False to indicate that the stream is not closed (as it will be
-        open for the duration of Airflow's lifecycle).
+        Returns False to indicate that the stream is not closed, as it will be
+        open for the duration of Airflow's lifecycle.
 
         For compatibility with the io.IOBase interface.
         """
         return False
 
     def _propagate_log(self, message):
-        """
-        Propagate message removing escape codes.
-        """
+        """Propagate message removing escape codes."""
         self.logger.log(self.level, remove_escape_codes(message))
 
     def write(self, message):
         """
         Do whatever it takes to actually log the specified logging record
+
         :param message: message to log
         """
         if not message.endswith("\n"):
@@ -121,15 +119,13 @@ class StreamLogWriter(object):
         else:
             self._buffer += message
             self._propagate_log(self._buffer.rstrip())
-            self._buffer = str()
+            self._buffer = ''
 
     def flush(self):
-        """
-        Ensure all logging output has been flushed
-        """
+        """Ensure all logging output has been flushed"""
         if len(self._buffer) > 0:
             self._propagate_log(self._buffer)
-            self._buffer = str()
+            self._buffer = ''
 
     def isatty(self):
         """
@@ -145,10 +141,12 @@ class RedirectStdHandler(StreamHandler):
     whatever sys.stderr/stderr is currently set to rather than the value of
     sys.stderr/stdout at handler construction time.
     """
+
     def __init__(self, stream):
-        if not isinstance(stream, six.string_types):
-            raise Exception("Cannot use file like objects. Use 'stdout' or 'stderr'"
-                            " as a str and without 'ext://'.")
+        if not isinstance(stream, str):
+            raise Exception(
+                "Cannot use file like objects. Use 'stdout' or 'stderr' as a str and without 'ext://'."
+            )
 
         self._use_stderr = True
         if 'stdout' in stream:
@@ -159,35 +157,17 @@ class RedirectStdHandler(StreamHandler):
 
     @property
     def stream(self):
+        """Returns current stream."""
         if self._use_stderr:
             return sys.stderr
 
         return sys.stdout
 
 
-@contextmanager
-def redirect_stdout(logger, level):
-    writer = StreamLogWriter(logger, level)
-    try:
-        sys.stdout = writer
-        yield
-    finally:
-        sys.stdout = sys.__stdout__
-
-
-@contextmanager
-def redirect_stderr(logger, level):
-    writer = StreamLogWriter(logger, level)
-    try:
-        sys.stderr = writer
-        yield
-    finally:
-        sys.stderr = sys.__stderr__
-
-
 def set_context(logger, value):
     """
     Walks the tree of loggers and tries to set the context for each handler
+
     :param logger: logger
     :param value: value to set
     """
