@@ -132,7 +132,7 @@ def execute_in_subprocess(cmd: List[str]):
     :param cmd: command and arguments to run
     :type cmd: List[str]
     """
-    log.info("Executing cmd: %s", " ".join([shlex.quote(c) for c in cmd]))
+    log.info("Executing cmd: %s", " ".join(shlex.quote(c) for c in cmd))
     with subprocess.Popen(
         cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, bufsize=0, close_fds=True
     ) as proc:
@@ -153,25 +153,30 @@ def execute_interactive(cmd: List[str], **kwargs):
     state after the process is completed e.g. if the subprocess hides the cursor, it will be restored after
     the process is completed.
     """
-    log.info("Executing cmd: %s", " ".join([shlex.quote(c) for c in cmd]))
+    log.info("Executing cmd: %s", " ".join(shlex.quote(c) for c in cmd))
 
     old_tty = termios.tcgetattr(sys.stdin)
     tty.setraw(sys.stdin.fileno())
 
     # open pseudo-terminal to interact with subprocess
-    master_fd, slave_fd = pty.openpty()
-    try:  # pylint: disable=too-many-nested-blocks
+    primary_fd, secondary_fd = pty.openpty()
+    try:
         # use os.setsid() make it run in a new process group, or bash job control will not be enabled
         with subprocess.Popen(
-            cmd, stdin=slave_fd, stdout=slave_fd, stderr=slave_fd, universal_newlines=True, **kwargs
+            cmd,
+            stdin=secondary_fd,
+            stdout=secondary_fd,
+            stderr=secondary_fd,
+            universal_newlines=True,
+            **kwargs,
         ) as proc:
             while proc.poll() is None:
-                readable_fbs, _, _ = select.select([sys.stdin, master_fd], [], [])
+                readable_fbs, _, _ = select.select([sys.stdin, primary_fd], [], [])
                 if sys.stdin in readable_fbs:
                     input_data = os.read(sys.stdin.fileno(), 10240)
-                    os.write(master_fd, input_data)
-                if master_fd in readable_fbs:
-                    output_data = os.read(master_fd, 10240)
+                    os.write(primary_fd, input_data)
+                if primary_fd in readable_fbs:
+                    output_data = os.read(primary_fd, 10240)
                     if output_data:
                         os.write(sys.stdout.fileno(), output_data)
     finally:
@@ -233,7 +238,7 @@ def patch_environ(new_env_variables: Dict[str, str]):
     """
     current_env_state = {key: os.environ.get(key) for key in new_env_variables.keys()}
     os.environ.update(new_env_variables)
-    try:  # pylint: disable=too-many-nested-blocks
+    try:
         yield
     finally:
         for key, old_value in current_env_state.items():
