@@ -34,6 +34,7 @@ from airflow.www.extensions.init_appbuilder_links import init_appbuilder_links
 from airflow.www.extensions.init_dagbag import init_dagbag
 from airflow.www.extensions.init_jinja_globals import init_jinja_globals
 from airflow.www.extensions.init_manifest_files import configure_manifest_files
+from airflow.www.extensions.init_robots import init_robots
 from airflow.www.extensions.init_security import init_api_experimental_auth, init_xframe_protection
 from airflow.www.extensions.init_session import init_airflow_session_interface, init_permanent_session
 from airflow.www.extensions.init_views import (
@@ -61,9 +62,7 @@ def sync_appbuilder_roles(flask_app):
     # will add the new Views and Menus names to the backend, but will not
     # delete the old ones.
     if conf.getboolean('webserver', 'UPDATE_FAB_PERMS'):
-        security_manager = flask_app.appbuilder.sm
-        security_manager.sync_roles()
-        security_manager.sync_resource_permissions()
+        flask_app.appbuilder.sm.sync_roles()
 
 
 def create_app(config=None, testing=False):
@@ -112,6 +111,8 @@ def create_app(config=None, testing=False):
 
     init_api_experimental_auth(flask_app)
 
+    init_robots(flask_app)
+
     Cache(app=flask_app, config={'CACHE_TYPE': 'filesystem', 'CACHE_DIR': '/tmp'})
 
     init_flash_views(flask_app)
@@ -141,7 +142,7 @@ def create_app(config=None, testing=False):
 
 def cached_app(config=None, testing=False):
     """Return cached instance of Airflow WWW app"""
-    global app  # pylint: disable=global-statement
+    global app
     if not app:
         app = create_app(config=config, testing=testing)
     return app
@@ -149,5 +150,5 @@ def cached_app(config=None, testing=False):
 
 def purge_cached_app():
     """Removes the cached version of the app in global state."""
-    global app  # pylint: disable=global-statement
+    global app
     app = None
