@@ -58,14 +58,14 @@ export const convertAndFormatUTC = (datetime, tz) => {
 };
 
 export const secondsToString = (seconds) => {
-  let numdays    = Math.floor((seconds % 31536000) / 86400);
-  let numhours   = Math.floor(((seconds % 31536000) % 86400) / 3600);
-  let numminutes = Math.floor((((seconds % 31536000) % 86400) % 3600) / 60);
-  let numseconds = Math.floor((((seconds % 31536000) % 86400) % 3600) % 60);
-  return (numdays > 0    ? numdays    + (numdays    === 1 ? ' day '    : ' days ')    : '') +
-         (numhours > 0   ? numhours   + (numhours   === 1 ? ' hour '   : ' hours ')   : '') +
-         (numminutes > 0 ? numminutes + (numminutes === 1 ? ' minute ' : ' minutes ') : '') +
-         (numseconds > 0 ? numseconds + (numseconds === 1 ? ' second'  : ' seconds')  : '');
+  const numdays = Math.floor((seconds % 31536000) / 86400);
+  const numhours = Math.floor(((seconds % 31536000) % 86400) / 3600);
+  const numminutes = Math.floor((((seconds % 31536000) % 86400) % 3600) / 60);
+  const numseconds = Math.floor((((seconds % 31536000) % 86400) % 3600) % 60);
+  return (numdays > 0 ? numdays + (numdays === 1 ? ' day ' : ' days ') : '')
+         + (numhours > 0 ? numhours + (numhours === 1 ? ' hour ' : ' hours ') : '')
+         + (numminutes > 0 ? numminutes + (numminutes === 1 ? ' minute ' : ' minutes ') : '')
+         + (numseconds > 0 ? numseconds + (numseconds === 1 ? ' second' : ' seconds') : '');
 };
 
 export function updateAllDateTimes() {
@@ -74,7 +74,10 @@ export function updateAllDateTimes() {
   $('time[data-datetime-convert!="false"]').each((_, el) => {
     const $el = $(el);
     const dt = moment($el.attr('datetime'));
-    $el.text(dt.format(defaultFormat));
+    // eslint-disable-next-line no-underscore-dangle
+    if (dt._isValid) {
+      $el.text(dt.format(defaultFormat));
+    }
     if ($el.attr('title') !== undefined) {
       // If displayed date is not UTC, have the UTC date in a title attribute
       $el.attr('title', dt.isUTC() ? '' : `UTC: ${dt.clone().utc().format()}`);
